<?php
// $Id: uc_product_keys.ca.inc,v 1.3 2009/12/16 14:24:02 freixas Exp $

/**
 * @file
 * This handles integration with Ubercart's conditional actions
 * module.
 */

/******************************************************************************
 * Conditional Actions Hooks                                                  *
 ******************************************************************************/

/**
 * Implementation of hook_ca_entity().
 *
 * @return The uc_product_keys entities.
 */
function uc_product_keys_ca_entity() {
  $entities['uc_product_keys'] = array(
    '#title' => t('Ubercart product key'),
    '#type' => 'array',
  );

  $entities['uc_product_keys_user'] = array(
    '#title' => t('Ubercart user'),
    '#type' => 'object',
  );

  $entities['uc_product_keys_order'] = array(
    '#title' => t('Order'),
    '#type' => 'object',
  );

  return $entities;
}

function uc_product_keys_ca_condition() {
  return array();
}

/**
 * Implementation of hook_ca_trigger().
 *
 * A "trigger" is really just an event. It doesn't trigger anything
 * unless the event meets all the conditions imposed on it.
 *
 * @return The uc_product_keys triggers.
 */
function uc_product_keys_ca_trigger() {
  $triggers['uc_product_keys_assigned'] = array(
    '#title' => t('Product keys are assigned to a customer'),
    '#category' => t('Product keys'),
    '#arguments' => array(
      'keys' => array(
        '#entity' => 'uc_product_keys',
        '#title' => t('Product keys purchased by a customer'),
	),
      'user' => array(
	'#entity' => 'uc_product_keys_user',
	'#title' => t('The customer who purchased the product keys'),
	),
      'order' => array(
	'#entity' => 'uc_product_keys_order',
	'#title' => t('The order in which the customer purchased the product keys'),
	),
      ),
    );

  return $triggers;
}

/**
 * Implementation of hook_ca_predicate().
 *
 * A "predicate" is just a confusing way of naming a collection of
 * conditions. To make it worse, it is also a collection of actions to
 * take when those conditions are met.
 *
 * Predicates tend to be named for the actions they collect, adding to
 * the confusion.
 *
 * Predicates are things a user might want to create based on
 * available triggers, conditions and actions. For convenience, we
 * create ones that might be of general use.
 *
 * @return The uc_product_keys predicates.
 */
function uc_product_keys_ca_predicate() {
  $predicates = array();
  $message_body = <<< EOF
<table width="95%" border="0" cellspacing="0" cellpadding="1" align="center" bgcolor="#006699" style="font-family: verdana, arial, helvetica; font-size: small;">
  <tr>
    <td>
      <table width="100%" border="0" cellspacing="0" cellpadding="5" align="center" bgcolor="#FFFFFF" style="font-family: verdana, arial, helvetica; font-size: small;">
        <tr valign="top">
          <td>
            <table width="100%" style="font-family: verdana, arial, helvetica; font-size: small;">
              <tr>
                <td>
                  [site-logo]
                </td>
                <td width="98%">
                  <div style="padding-left: 1em;">
                  <span style="font-size: large;">[store-name]</span><br />
                  [site-slogan]
                  </div>
                </td>
                <td nowrap="nowrap">
                  [store-address]<br />[store-phone]
                </td>
              </tr>
            </table>
          </td>
        </tr>
        <tr valign="top">
          <td>
	    <p><b>Thanks for your order, [order-first-name]! Here are your product keys:</b></p>
	   [product-keys]
	   <p><b>[store-link]</b><br /><b>[site-slogan]</b></p>
	   <p>You may also view all your product keys by visiting your
	      <a href="[site-url][account-url]">account</a> and selecting the "Product keys" link.</p>
	  </td>
	</tr>
      </table>
    </td>
  </tr>
</table>
EOF;

  $predicates['uc_product_keys_update_email_customer'] = array(
    '#title' => t('E-mail product keys to a customer'),
    '#description' => t('E-mail product keys to the customer who purchased them.'),
    '#class' => 'notification',
    '#status' => 1,
    '#trigger' => 'uc_product_keys_assigned',
    '#actions' => array(
      array(
        '#name' => 'uc_product_keys_email',
        '#title' => t('E-mail product keys to a user'),
	'#argument_map' => array(
	  'keys' => 'keys',
	  'user' => 'user',
	  'order' => 'order',
	  ),
        '#settings' => array(
          'from' => uc_store_email_from(),
          'addresses' => '[mail]',
          'subject' => t('Your product keys'),
          'message' => $message_body,
          'format' => 2,
        ),
      ),
    ),
  );

  return $predicates;
}

/**
 * Implementation of hook_ca_action().
 *
 * This is something we should do if a trigger (event) fires and all
 * its conditions are met.
 *
 * @return The uc_product_keys actions.
 */
function uc_product_keys_ca_action() {
  $keys_arg = array(
    '#entity' => 'uc_product_keys',
    '#title' => t('Product keys'),
  );

  $actions['uc_product_keys_email'] = array(
    '#title' => t('E-mail product keys to a user'),
    '#category' => t('Product keys'),
    '#callback' => 'uc_product_keys_action_email',
    '#arguments' => array(
      'keys' => array(
	'#entity' => 'uc_product_keys',
	'#title' => t('Product keys'),
	),
      'user' => array(
	'#entity' => 'uc_product_keys_user',
	'#title' => t('Product keys user'),
	),
      'order' => array(
	'#entity' => 'uc_product_keys_order',
	'#title' => t('Product keys order'),
	),
      ),
    );

  return $actions;
}

/******************************************************************************
 * Action Callbacks and Forms                                                 *
 ******************************************************************************/

/**
 * Email product keys to a customer
 *
 * The recipients, subject, and message fields take product key token replacements.
 *
 * @param $keys The user's product keys.
 * @param $user The user.
 * @param $order The order in which the keys were purchased.
 * @param $settings An array containg these keys: 'from', 'addresses','subject', 'message', and 'format'.
 *	'format' is the id of an input format to use to process the message.
 * @see uc_product_keys_ca_predicate()
 * @see uc_product_keys_action_email_form()
 */
function uc_product_keys_action_email($keys, $user, $order, $settings) {
  $account = user_load($user->uid);

  // Token replacements for the subject and body
  $settings['replacements'] = array(
    'global' => NULL,
    'product-keys' => $keys,
    'order' => $order,
    'user' => $account,
  );

  // Apply token replacements to from and recipient e-mail addressses.
  $from = token_replace_multiple($settings['from'], $settings['replacements']);
  $addresses = token_replace_multiple($settings['addresses'], $settings['replacements']);

  // Split up our recipient e-mail addresses.
  $recipients = array();
  foreach (explode("\n", $addresses) as $address) {
    $recipients[] = trim($address);
  }

  foreach ($recipients as $email) {
    $sent = drupal_mail(
      'uc_product_keys',				// Module
      'action-mail',					// Key
      $email,						// To
      uc_store_mail_recipient_language($email),		// Language
      $settings,					// Params
      empty($settings['from']) ? uc_store_email_from() : $settings['from'] // From
      );

    if (!$sent['result']) {
      watchdog(
	'ca',
	'Attempt to e-mail @email concerning product keys for order @order_id failed.',
	array('@email' => $email, '@order_id' => $order->order_id), WATCHDOG_ERROR);
    }
  }
}

/**
 * Here's the code that performs the email action.
 *
 * @param $form_state The form state.
 * @param $settings An optional array containg these keys: 'from', 'addresses','subject', 'message', and 'format'.
 *	'format' is the id of an input format to use to process the message.
 * @see uc_product_keys_ca_predicate()
 * @see uc_product_keys_action_email()
 */
function uc_product_keys_action_email_form($form_state, $settings = array()) {
  $form['from'] = array(
    '#type' => 'textfield',
    '#title' => t('Sender'),
    '#default_value' => isset($settings['from']) ? $settings['from'] : uc_store_email_from(),
    '#description' => t('The "From" address.'),
    '#required' => TRUE,
  );
  $form['addresses'] = array(
    '#type' => 'textarea',
    '#title' => t('Recipients'),
    '#default_value' => isset($settings['addresses']) ? $settings['addresses'] : '[order-email]',
    '#description' => t('Enter the email addresses to receive the notifications, one on each line. You may use order tokens for dynamic email addresses.'),
    '#required' => TRUE,
  );
  $form['subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#default_value' => $settings['subject'],
    '#required' => TRUE,
  );
  $form['message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message'),
    '#default_value' => $settings['message'],
  );

  // We add the #is_format element to allow us to locate and configure the filters.
  $form['format'] = filter_form($settings['format']) + array('#is_format' => TRUE);

  $form['token_help'] = array(
    '#type' => 'fieldset',
    '#title' => t('Replacement patterns'),
    '#description' => t('You can make use of the replacement patterns in the e-mail from and recipients fields, the subject, and the message body.'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  foreach (array('global', 'order', 'user', 'product keys') as $name) {
    $form['token_help'][$name] = array(
      '#type' => 'fieldset',
      '#title' => t('@name replacement patterns', array('@name' => drupal_ucfirst($name))),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $form['token_help'][$name]['containsent'] = array(
      '#value' => theme('token_help', $name),
    );
  }

  return $form;
}
